@@ -24,7 +24,7 @@ import serial
 import re
 import time
 
-deviceClassNames = ("TecpelDMM8061", "VoltcraftVC820", "VoltcraftVC840",
+deviceClassNames = ("TecpelDMM8061", "VoltcraftVC840",
     "XLS200", "KernPCB", "BS600")
 
 class Device(object):
@@ -373,6 +373,7 @@ class TecpelDMM8061(Device):
         hertz = 0
         volt = 0
         ampere = 0
+        celsius = 0
 
 
         time = None
@@ -393,18 +394,26 @@ class TecpelDMM8061(Device):
 
         @property
         def unit(self):
+            r = ""
+            if self.delta:
+                r += u"Δ"
+
             if self.ohm:
-                return u"Ω"
+                return r + u"Ω"
             elif self.farad:
-                return "F"
+                return r + "F"
             elif self.hertz:
-                return "Hz"
+                return r + "Hz"
             elif self.volt:
-                return "V"
+                return r + "V"
             elif self.ampere:
-                return "A"
+                return r + "A"
+            elif self.celsius:
+                return r + u"°C"
+            elif self.duty_cycle:
+                return r + "%"
             else:
-                return u"°C"
+                return r + ""
 
         @property
         def factor(self):
@@ -457,7 +466,7 @@ class TecpelDMM8061(Device):
             if idList[startIndex + 1] == 2 and idList[startIndex + 4] == 5 and idList[startIndex + 7] == 8 and\
                     idList[startIndex + 10] == 11 and idList[startIndex + 12] == 13:
                 
-                valueString = read[startIndex:(startIndex+13)]
+                valueString = read[startIndex:(startIndex+14)]
 
                 result = self.__Value()
 
@@ -539,6 +548,9 @@ class TecpelDMM8061(Device):
                 result.volt = self._testBit(b, 2)
                 result.ampere = self._testBit(b, 3)
 
+                b = ord(valueString[13])
+                result.celsius = self._testBit(b, 0)
+
                 result.time = time.time()
 
                 return result
@@ -549,14 +561,6 @@ class TecpelDMM8061(Device):
         return None
 
 
-class VoltcraftVC820(TecpelDMM8061):
-    """This class represents a VOLTCRAFT VC 820 multimeter
-
-    The VC 820 is identical to the TecpelDMM8061.
-
-    """
-
-
 class VoltcraftVC840(TecpelDMM8061):
     """This class represents a VOLTCRAFT VC 840 multimeter
 
