@@ -116,16 +116,11 @@ class SettingsDialog(QtGui.QDialog):
 
 class DisplayWidget(QtGui.QWidget):
     """Widget containing a lcd display and buttons to modify or delete a device."""
-    calibrationType = 1 # 0: two values calibration, 1: slope and intercept calibration
+    calibrationType = 1 # 0 = two values calibration, 1 = slope and intercept calibration
     twoValueCalibration = (0.0, 0.0), (1.0, 1.0)
     slopeInterceptCalibration = 1.0, 0.0
 
-    siNames = si.getSiNames("normal")
-
-    is1PrefixIndex = siNames.index("") # QComboBox Index
-    should1PrefixIndex = siNames.index("") # QComboBox Index
-    is2PrefixIndex = siNames.index("") # QComboBox Index
-    should2PrefixIndex = siNames.index("") # QComboBox Index
+    indexes = {}
 
     is1 = 0
     should1 = 0
@@ -149,6 +144,7 @@ class DisplayWidget(QtGui.QWidget):
         qfile = QtCore.QFile(":/ui/displayWidget.ui")
         qfile.open(QtCore.QIODevice.ReadOnly)
         self.ui = uic.loadUi(qfile, self)
+        self.setObjectName(str(deviceID))
 
         self.deviceID = deviceID
         self.dm = dm
@@ -163,77 +159,33 @@ class DisplayWidget(QtGui.QWidget):
     def deviceSettings(self):
         """Open a DeviceSettingsDialog."""
 
-        popup = DeviceSettingsDialog(self.deviceID, self.dm)
+        popup = DeviceSettingsDialog(self, self.dm)
+        popup.show()
 
-        popup.slope.setValue(self.slopeInterceptCalibration[0])
-        popup.intercept.setValue(self.slopeInterceptCalibration[1])
-        popup.is1.setValue(self.is1)
-        popup.should1.setValue(self.should1)
-        popup.is2.setValue(self.is2)
-        popup.should2.setValue(self.should2)
+    def update(self, rv):
+        self.rv = rv
+        self.crv = self.dm.calibrate(self.rv, self.calibration, self.unit)
 
-        # Add prefixes to combobox
-        for i in [popup.is1Prefix, popup.should1Prefix, popup.is2Prefix, popup.should2Prefix]:
-            i.clear()
-            i.addItems(self.siNames)
-
-        popup.is1Prefix.setCurrentIndex(self.is1PrefixIndex)
-        popup.should1Prefix.setCurrentIndex(self.should1PrefixIndex)
-        popup.is2Prefix.setCurrentIndex(self.is2PrefixIndex)
-        popup.should2Prefix.setCurrentIndex(self.should2PrefixIndex)
-
-        if self.calibrationType == 1:
-            popup.slopeInterceptButton.setChecked(True)
-        elif self.calibrationType == 0:
-            popup.valuesButton.setChecked(True)
-
-        popup.unit.setText(self.unit)
-
-        popup.exec_()
-
-        self.twoValueCalibration = popup.twoValueCalibration
-        self.slopeInterceptCalibration = popup.slopeInterceptCalibration
-
-        self.is1 = popup.is1.value()
-        self.should1 = popup.should1.value()
-        self.is2 = popup.is2.value()
-        self.should2 = popup.should2.value()
-
-        self.is1PrefixIndex = popup.is1Prefix.currentIndex()
-        self.should1PrefixIndex = popup.should1Prefix.currentIndex()
-        self.is2PrefixIndex = popup.is2Prefix.currentIndex()
-        self.should2PrefixIndex = popup.should2Prefix.currentIndex()
-
-        if popup.slopeInterceptButton.isChecked():
-            self.calibrationType = 1
-            self.calibration = self.slopeInterceptCalibration
-        elif popup.valuesButton.isChecked():
-            self.calibrationType = 0
-            self.calibration = self.twoValueCalibration
-
-        self.unit = popup.unit.text()
-
-    def delete(self):
-        """Delete Widget."""
-
-        self.deleteLater()
+        self.lcdNumber.display(self.crv.value)
+        self.label.setText(self.crv.prefix + self.crv.unit)
 
     def close(self):
         """Close the device."""
 
         self.dm.closeDevice(self.deviceID)
+        self.deleteLater()
 
 
 class DeviceSettingsDialog(QtGui.QDialog):
     """Settings dialog for device specific settings."""
 
-    def __init__(self, deviceID, dm, parent=None):
+    def __init__(self, parentDisplayWidget, deviceManager, parent=None):
         """
 
-        :param deviceID: DeviceID
-        :type deviceID: DeviceID
-        :param dm: DeviceManager
-        :type dm: DeviceManager
+        :param parentDisplayWidget: parent/corresponding DisplayWidget
+        :type parentDisplayWidget: DisplayWidget
+        :param deviceManager: DeviceManager
+        :type deviceManager: DeviceManager
 
         """
         QtGui.QDialog.__init__(self, parent)
@@ -241,8 +193,8 @@ class DeviceSettingsDialog(QtGui.QDialog):
         qfile.open(QtCore.QIODevice.ReadOnly)
         self.ui = uic.loadUi(qfile, self)
 
-        self.deviceID = deviceID
-        self.dm = dm
+        self.parent = parentDisplayWidget
+        self.dm = deviceManager
 
         self.settings = QtCore.QSettings()
 
@@ -251,34 +203,92 @@ class DeviceSettingsDialog(QtGui.QDialog):
         self.saveButton.clicked.connect(self.save)
         self.loadButton.clicked.connect(self.load)
 
+        self.buttonBox.accepted.connect(self.accepted)
+
         self.timer = QtCore.QTimer()
         self.timer.timeout.connect(self.update)
         self.timer.start(50)
 
         self.finished.connect(self._finish)
 
+        self.setUp()
+
     def _finish(self):
         self.timer.stop()
 
+    def setUp(self):
+        self.slope.setValue(self.parent.slopeInterceptCalibration[0])
+        self.intercept.setValue(self.parent.slopeInterceptCalibration[1])
+        self.is1.setValue(self.parent.is1)
+        self.should1.setValue(self.parent.should1)
+        self.is2.setValue(self.parent.is2)
+        self.should2.setValue(self.parent.should2)
+
+        # Add prefixes to combobox
+        for box, name in [(self.is1Prefix, "is1Prefix"),
+                (self.should1Prefix, "should1Prefix"),
+                (self.is2Prefix, "is2Prefix"),
+                (self.should2Prefix, "should2Prefix"),
+                (self.slopePrefix, "slopePrefix"),
+                (self.interceptPrefix, "interceptPrefix")]:
+            box.clear()
+            box.addItems(si.getSiNames("normal"))
+            box.setCurrentIndex(self.parent.indexes.get(name,
+                si.getSiNames("normal").index("")))
+
+        if self.parent.calibrationType == 1:
+            self.slopeInterceptButton.setChecked(True)
+        elif self.parent.calibrationType == 0:
+            self.valuesButton.setChecked(True)
+
+        self.unit.setText(self.parent.unit)
+
+    def accepted(self):
+        self.parent.twoValueCalibration = self.twoValueCalibration
+        self.parent.slopeInterceptCalibration = self.slopeInterceptCalibration
+
+        self.parent.is1 = self.is1.value()
+        self.parent.should1 = self.should1.value()
+        self.parent.is2 = self.is2.value()
+        self.parent.should2 = self.should2.value()
+
+        for box, name in [(self.is1Prefix, "is1Prefix"),
+                (self.should1Prefix, "should1Prefix"),
+                (self.is2Prefix, "is2Prefix"),
+                (self.should2Prefix, "should2Prefix"),
+                (self.slopePrefix, "slopePrefix"),
+                (self.interceptPrefix, "interceptPrefix")]:
+            self.parent.indexes[name] = box.currentIndex()
+
+        if self.slopeInterceptButton.isChecked():
+            self.parent.calibrationType = 1
+            self.parent.calibration = self.parent.slopeInterceptCalibration
+        elif self.valuesButton.isChecked():
+            self.parent.calibrationType = 0
+            self.parent.calibration = self.parent.twoValueCalibration
+
+        self.parent.unit = self.unit.text()
+
     def update(self):
         self.twoValueCalibration = ((self.is1.value() * si.getFactor(str(self.is1Prefix.currentText())),
                 self.should1.value()  * si.getFactor(str(self.should1Prefix.currentText()))),
             (self.is2.value() * si.getFactor(str(self.is2Prefix.currentText())), 
                 self.should2.value() * si.getFactor(str(self.should2Prefix.currentText()))))
 
-        self.slopeInterceptCalibration = self.slope.value(), self.intercept.value()
+        self.slopeInterceptCalibration = (self.slope.value() * si.getFactor(str(self.slopePrefix.currentText())),
+            self.intercept.value() * si.getFactor(str(self.interceptPrefix.currentText())))
         
         if self.slopeInterceptButton.isChecked():
             self.calibration = self.slopeInterceptCalibration
         elif self.valuesButton.isChecked():
             self.calibration = self.twoValueCalibration
 
-        crv = self.dm.getCalibratedLastRawValue(self.deviceID, self.calibration, self.unit.text())
-        self.calibratedLabel.setText(u"{:n} {}{}".format(crv.value, crv.prefix, crv.unit))
-
-        rv = self.dm.getLastRawValue(self.deviceID)
+        rv = self.parent.rv
         self.normalLabel.setText(u"{:n} {}{}".format(rv.value, rv.prefix, rv.unit))
 
+        crv = self.dm.calibrate(rv, self.calibration, self.unit.text())
+        self.calibratedLabel.setText(u"{:n} {}{}".format(crv.value, crv.prefix, crv.unit))
+
 
     def save(self):
         self.settings.setValue("calibration/" +
@@ -305,6 +315,12 @@ class DeviceSettingsDialog(QtGui.QDialog):
         self.settings.setValue("calibration/" +
             self.slotComboBox.currentText() + "should2PrefixIndex", 
             self.should2Prefix.currentIndex())
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "slopePrefixIndex", 
+            self.slopePrefix.currentIndex())
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "interceptPrefixIndex", 
+            self.interceptPrefix.currentIndex())
 
         self.settings.setValue("calibration/" +
             self.slotComboBox.currentText() + "slope",
@@ -318,8 +334,7 @@ class DeviceSettingsDialog(QtGui.QDialog):
             self.unit.text())
 
     def load(self):
-        allItems = [self.is1Prefix.itemText(i) for i in range(self.is1Prefix.count())]
-        emptyStringIndex = allItems.index("")
+        standardIndex = si.getSiNames("normal").index("")
 
         self.is1.setValue(self.settings.value("calibration/" +
             self.slotComboBox.currentText() + "is1", 0).toInt()[0])
@@ -330,13 +345,17 @@ class DeviceSettingsDialog(QtGui.QDialog):
         self.should2.setValue(self.settings.value("calibration/" +
             self.slotComboBox.currentText() + "should2", 1).toInt()[0])
         self.is1Prefix.setCurrentIndex(self.settings.value("calibration/" +
-            self.slotComboBox.currentText() + "is1PrefixIndex", emptyStringIndex).toInt()[0])
+            self.slotComboBox.currentText() + "is1PrefixIndex", standardIndex).toInt()[0])
         self.should1Prefix.setCurrentIndex(self.settings.value("calibration/" +
-            self.slotComboBox.currentText() + "should1PrefixIndex", emptyStringIndex).toInt()[0])
+            self.slotComboBox.currentText() + "should1PrefixIndex", standardIndex).toInt()[0])
         self.is2Prefix.setCurrentIndex(self.settings.value("calibration/" +
-            self.slotComboBox.currentText() + "is2PrefixIndex", emptyStringIndex).toInt()[0])
+            self.slotComboBox.currentText() + "is2PrefixIndex", standardIndex).toInt()[0])
         self.should2Prefix.setCurrentIndex(self.settings.value("calibration/" +
-            self.slotComboBox.currentText() + "should2PrefixIndex", emptyStringIndex).toInt()[0])
+            self.slotComboBox.currentText() + "should2PrefixIndex", standardIndex).toInt()[0])
+        self.slopePrefix.setCurrentIndex(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "slopePrefixIndex", standardIndex).toInt()[0])
+        self.interceptPrefix.setCurrentIndex(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "interceptPrefixIndex", standardIndex).toInt()[0])
 
         self.slope.setValue(self.settings.value("calibration/" +
             self.slotComboBox.currentText() + "slope", 1).toInt()[0])
@@ -350,7 +369,7 @@ class DeviceSettingsDialog(QtGui.QDialog):
             self.slotComboBox.currentText() + "unit", "").toString())
 
     def setCurrentValue1(self):
-        rv = self.dm.getLastRawValue(self.deviceID)
+        rv = self.parent.rv
         self.is1.setValue(rv.value)
 
         index = self.is1Prefix.findText(rv.prefixName)
@@ -358,7 +377,7 @@ class DeviceSettingsDialog(QtGui.QDialog):
         self.should1Prefix.setCurrentIndex(index)
 
     def setCurrentValue2(self):
-        rv = self.dm.getLastRawValue(self.deviceID)
+        rv = self.parent.rv
         self.is2.setValue(rv.value)
 
         index = self.is1Prefix.findText(rv.prefixName)
@@ -370,7 +389,6 @@ class MainWindow(QtGui.QMainWindow):
     """Main window."""
 
     dm = None
-    displayWidgets = {}
 
     log = False
     tmpfile = None
@@ -380,6 +398,8 @@ class MainWindow(QtGui.QMainWindow):
 
     officePath = None
 
+    latestValues = {}
+
     def __init__(self, parent=None):
         QtGui.QMainWindow.__init__(self, parent)
         qfile = QtCore.QFile(":/ui/mainWindow.ui")
@@ -454,35 +474,32 @@ class MainWindow(QtGui.QMainWindow):
                     xls200ID = self.dm.openDevice(device, port)
 
                     if xls200ID:
-                        
+
+                        # Index 0 --> "No device"
                         if xls200Popup.subdevice1ComboBox.currentIndex() != 0:
                             sub1 = unicode(xls200Popup.subdevice1ComboBox.currentText())
                             deviceID = self.dm.openSubdevice(sub1, xls200ID, 1)
                             sub1Widget = DisplayWidget(deviceID, self.dm)
                             self.verticalLayout.addWidget(sub1Widget)
-                            self.displayWidgets[deviceID] = sub1Widget
 
                         if xls200Popup.subdevice2ComboBox.currentIndex() != 0:
                             sub2 = unicode(xls200Popup.subdevice2ComboBox.currentText())
                             deviceID = self.dm.openSubdevice(sub2, xls200ID, 2)
                             sub2Widget = DisplayWidget(deviceID, self.dm)
                             self.verticalLayout.addWidget(sub2Widget)
-                            self.displayWidgets[deviceID] = sub2Widget
 
                         if xls200Popup.subdevice3ComboBox.currentIndex() != 0:
                             sub3 = unicode(xls200Popup.subdevice3ComboBox.currentText())
                             deviceID = self.dm.openSubdevice(sub3, xls200ID, 3)
                             sub3Widget = DisplayWidget(deviceID, self.dm)
                             self.verticalLayout.addWidget(sub3Widget)
-                            self.displayWidgets[deviceID] = sub3Widget
 
             else:
                 deviceID = self.dm.openDevice(device, port)
 
-                if deviceID:
+                if deviceID != None:
                     deviceWidget = DisplayWidget(deviceID, self.dm)
                     self.verticalLayout.addWidget(deviceWidget)
-                    self.displayWidgets[deviceID] = deviceWidget
 
     def startStopLogging(self):
         """Start/Stop logging."""
@@ -531,37 +548,21 @@ class MainWindow(QtGui.QMainWindow):
     def update(self):
         """Update the displayWidgets and, when logging enabled log."""
         
-        # python3 incompatibility: .iteritems()
-        # Delete unnecessary widgets
-        deviceIDsToBeDeleted = []
-        for deviceID, widget in self.displayWidgets.iteritems():
-            if deviceID not in self.dm.getAllDeviceIDs():
-                deviceIDsToBeDeleted.append(deviceID)
-        
         self.dm.closeEmptyMultiboxDevices()
 
-        for i in deviceIDsToBeDeleted:
-            self.displayWidgets[i].delete()
-            del(self.displayWidgets[i])
-
         # update widgets
-        for deviceID, widget in self.displayWidgets.iteritems():
-            unit = None
-            if widget.unit != "":
-                unit = widget.unit
+        while not self.dm.queue.empty():
+            deviceID, rv = self.dm.queue.get_nowait()
+            widget = self.findChild(DisplayWidget,str(deviceID))
+            if widget != 0 and widget != None:
+                widget.update(rv)
 
-            rv = self.dm.getCalibratedLastRawValue(widget.deviceID, widget.calibration, unit)
-            widget.lcdNumber.display(rv.value)
-            widget.label.setText(rv.prefix + rv.unit)
+            self.latestValues[deviceID] = rv
         
         # log    
         if self.log and ((time.time() - self.lasttime) > self.loggingInterval.value()):
-            for deviceID, widget in self.displayWidgets.iteritems():
-                unit = None
-                if widget.unit != "":
-                    unit = widget.unit
-                rv = self.dm.getCalibratedLastRawValue(widget.deviceID, widget.calibration, unit)
-                self.tmpfile.write("{:n}".format(rv.completeValue) + ";")
+            for widget in self.findChildren(DisplayWidget):
+                self.tmpfile.write("{:n}".format(widget.crv.completeValue) + ";")
 
             self.tmpfile.write("\n")
             self.lasttime = time.time()
